# Suspect Blocker  

**Description:**  

The **Suspect Blocker** module is a security tool designed to protect your Drupal website by detecting and blocking suspicious behavior, such as brute force or flood attacks, based on rapid or repeated access attempts. The module monitors all requests that result in suspicious status codes (e.g., 403, 404) or other unusual patterns and takes proactive measures to ban offending IPs when burst behavior is detected.

## Features:

1. **Burst Detection:**
   - Identifies and tracks rapid access attempts ("bursts") to multiple pages within a configurable time window.
   - Monitors requests resulting in specific status codes, such as 403 (Forbidden) and 404 (Not Found).

2. **IP Banning:**
   - Automatically bans IPs that exceed a configurable threshold of suspicious requests within the defined time window.
   - Integration with the Drupal Ban module for managing blocked IPs.

3. **Customizable Settings:**
   - Administrators can configure the following parameters through an intuitive settings page:
     - **Ban Threshold:** Number of suspicious requests required to trigger a ban.
     - **Time Window:** The period (in seconds) to monitor for bursts of suspicious activity.

4. **Real-Time Logging:**
   - Logs all suspicious access attempts for analysis and debugging.
   - Provides detailed information, including the IP address, the accessed path, and the HTTP status code.

5. **Efficient Resource Usage:**
   - Uses an in-memory logging system with automatic cleanup of outdated records to minimize database overhead.

6. **Seamless Integration:**
   - Fully compatible with Drupal core and leverages the Ban module for IP management.

## Use Cases:

- **Brute Force Attack Prevention:** Blocks bots or malicious users attempting to guess administrative credentials through repeated failed login attempts.
- **Flood Attack Mitigation:** Detects and blocks bots trying to overload your server with rapid requests to multiple pages.
- **Custom Security Rules:** Allows monitoring of specific HTTP status codes or paths to detect targeted attacks.

## Configuration:

1. Navigate to the module’s settings page: **`/admin/config/security/suspect-blocker`**.
2. Customize the following:
   - **Ban Threshold:** Set the maximum number of suspicious requests before banning an IP (default: 5).
   - **Time Window:** Define the time window (in seconds) for monitoring rapid requests (default: 60).
3. Save the settings, and the module will start monitoring your site for suspicious activity.

## Requirements:

- Drupal 10 or 11.
- Optional: Ban module (for IP blocking).

## Benefits:

- **Enhanced Security:** Adds an extra layer of protection against malicious traffic.
- **Easy Configuration:** No coding required—administrators can tweak security thresholds via the UI.
- **Performance Optimized:** Minimal impact on server performance with efficient in-memory data handling.
- **Scalable:** Suitable for small to large Drupal websites.

## How to contribute
If you want to contribute to this project, there is an easy way to do. 

First, install a fresh instance of Drupal using DDEV. You can find the instructions at https://www.drupal.org/docs/getting-started/installing-drupal/install-drupal-using-ddev#s-option-2-using-git-with-clone

Install the Suspect Blocker module
```bash
ddev composer require drupal/suspect_blocker
```

Enable the module using Drupal Admin UI or Drush
```bash
ddev drush pm:enable suspect_blocker
```

With that you should have a fresh Drupal installation with the Suspect Blocker module enabled.

Apply the changes and tests by editing the module files. You can find the module files at `modules/contrib/suspect_blocker`.

## How to run the tests
Run the following command to SSH into DDEV container
```bash
ddev ssh
```

Run the following command to run the tests
```bash
./vendor/bin/phpunit -c modules/contrib/suspect_blocker/phpunit.xml.dist
```

## How to send your changes for review

Create the issue at https://www.drupal.org/project/issues/suspect_blocker

Fork the project

Push your code

Create the Merge Request.